﻿using System;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Markup;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using Steema.TeeChart.WPF;
using Steema.TeeChart.WPF.Drawing;
using Steema.TeeChart.WPF.Export;
using Steema.TeeChart.WPF.Styles;
using Steema.TeeChart.WPF.Tools;

namespace GraphExport
{
    /// <summary>
    ///     Interaction logic for GraphView.xaml
    /// </summary>
    public partial class GraphView
    {
        private const string xamlFile = "GraphTempXamlFile.Xaml";
        private const string imageFile = "GraphTempImage.bmp";
        private DrawLine _drawLine1;
        private string _imageFilePath;
        private Line _lineSeries1;
        private string _xamlFilePath;
        private readonly double[] xArray = new double[100];
        private readonly double[] yArray = new double[100];

        public GraphView()
        {
            InitializeComponent();
            SetFilePath();
            DrawLine();
        }

        private void SetFilePath()
        {
            var dir = Environment.CurrentDirectory;

            _xamlFilePath = Path.Combine(dir, xamlFile);
            _imageFilePath = Path.Combine(dir, imageFile);
        }

        private void InitChart()
        {
            TChartControl.Series.Clear();
            TChartControl.Aspect.View3D = false;
            TChartControl.Header.Text = "Test for Tool DragLine";
            TChartControl.AutoRepaint = true;


            _lineSeries1 = new Line();
            _lineSeries1.Color = Colors.Blue;
            _lineSeries1.ColorEach = false;
            _lineSeries1.LinePen.Color = Colors.Green;
            _lineSeries1.Title = "lineSeries1";
            _lineSeries1.XValues.DataMember = "X";
            _lineSeries1.XValues.Order = ValueListOrder.Ascending;
            _lineSeries1.YValues.DataMember = "Y";
        }

        private void DrawLine()
        {
            btnDrawLine.IsEnabled = false;
            btnDstraightrawLine.IsEnabled = true;

            InitChart();

            for (var i = 0; i < 100; i++)
            {
                xArray[i] = i + 0.5;
             //   yArray[i] = i % 2 == 0 ? i + 0.5 : i - 0.5;
                yArray[i] = i % 2 == 0 ? i + 0.6 : i - 0.4;
            }

            _lineSeries1.Add(xArray, yArray);

            TChartControl.Series.Add(_lineSeries1);
        }

        private void DrawstraightLine()
        {
            btnDrawLine.IsEnabled = true;
            btnDstraightrawLine.IsEnabled = false;

            InitChart();

            for (var i = 0; i < 100; i++)
            {
                xArray[i] = i + 0.5;
                yArray[i] = 2;
            }

            _lineSeries1.Add(xArray, yArray);

            TChartControl.Series.Add(_lineSeries1);
        }

        private void ButtonDrawLineClick(object sender, RoutedEventArgs e)
        {
            DrawLine();
        }

        private void ButtonDrawstraightLineClick(object sender, RoutedEventArgs e)
        {
            DrawstraightLine();
        }

        private void ButtonExportClick(object sender, RoutedEventArgs e)
        {
            var xaml = new MyXAMLFormat(TChartControl.Chart);

            xaml.Save(_xamlFilePath);
            TChartControl.Export.Image.Bitmap.Save(_imageFilePath);


            var win = new Window1();
            object canvas;
            using (var fs = new FileStream(_xamlFilePath, FileMode.Open, FileAccess.Read))
            {
                canvas = XamlReader.Load(fs);
            }

            Application.Current.MainWindow.Hide();
            win.XamlPalceHolder.Content = canvas;
            var bimg = BitmapFromUri(new Uri(_imageFilePath));
            win.ImagePalceHolder.Source = bimg;
            win.ShowDialog();
            Application.Current.MainWindow.Show();
        }

        public static BitmapImage BitmapFromUri(Uri source)
        {
            var bitmap = new BitmapImage();
            bitmap.BeginInit();
            bitmap.UriSource = source;
            bitmap.CacheOption = BitmapCacheOption.OnLoad;
            bitmap.EndInit();
            return bitmap;
        }

        private void ButtonRunTimeClick(object sender, RoutedEventArgs e)
        {
            var win = new Window1();
            Application.Current.MainWindow.Hide();
            win.XamlPalceHolder.Content = new XamlExport();

            win.ShowDialog();
            Application.Current.MainWindow.Show();
        }
    }

    public class MyXAMLFormat : XAMLFormat
    {
        private readonly Chart chart;

        public MyXAMLFormat(Chart c)
            : base(c)
        {
            chart = c;
        }

        public override void Save(Stream stream)
        {
            var oldGraphics = chart.Graphics3D;

            try
            {
                chart.Graphics3D = new MyGraphics3DXAML(stream, chart, false);
                chart.Draw(null, new Rect(0, 0, Width, Height));

                var parent = AdornerLayer.GetAdornerLayer(chart.Parent.GetControl());
                if (parent != null)
                {
                    var adorners = parent.GetAdorners(chart.Parent.GetControl());
                    if (adorners != null)
                    {
                        foreach (TeeAdorner item in adorners)
                        {
                            item.Draw(chart.Graphics3D);
                        }
                    }
                }


                ((MyGraphics3DXAML)chart.Graphics3D).AddToStreamPublic("</Canvas>");
            }
            finally
            {
                chart.Graphics3D = oldGraphics;
            }
        }
    }

    public class MyGraphics3DXAML : Graphics3DXAML
    {
        public MyGraphics3DXAML(Stream stream, Chart chart, bool doShowImage)
            : base(stream, chart, doShowImage)
        {
        }

        public void AddToStreamPublic(string s)
        {
            AddToStream(s);
        }
    }
}